#!/usr/bin/perl
#
# Transcode Recording files to MP3 format
# so they can be streamed by streaming server
#

use strict;
use Data::Dumper;
use File::Find;
use File::Copy;
use File::Path;
use File::Basename;

my @ast_format = (
	"ulaw",
	"alaw",
	"gsm",
	"slin",
	"g722",
	"g729"
);

my %sox_format = (
	"ulaw" => "ul",
	"alaw" => "al"
);

#my %ffmpeg_format = (
#	"ulaw" => "mulaw",
#	"alaw" => "alaw"
#);

my %options;
my $single_file;
my $server_id;

#if ($ARGV[0] eq "-s") {
#    exit_trans_recordings();
#}

if(@ARGV == 2)
{
	($single_file, $server_id) = @ARGV;
}

get_directories_from_conf() or die "Didn't find any directory, check config file\n";

if(defined($single_file))
{
	print "Translate single file: $single_file\n";
	do_single_trans($single_file, $server_id);
	print "Done\n";
	exit;
}
else
{
#	my $pid = fork();
#	if(not defined $pid) {
#		die "Fork failed\n";
#	} elsif (0 == $pid) { # child
		open(my $fh, '>', "/var/run/trans_recordings.pid") or die $!;
		print $fh "$$";
		close($fh);

		print "Trancoding process [$$] started.\n";	

		while(1) {
			traverse_directories();
			sleep(60);
		}
#	}
}

sub exit_trans_recordings
{
	open(my $fh, '<', "/var/run/trans_recordings.pid") or die "Trans_recordings script is not running\n";

	my $pid = <$fh>;
	close($fh);

	# kill server
	kill 9, $pid;

	# clean up proccess
	my $exit_code = waitpid($pid, 0);
	print "Trans_recordings process terminated.\n";
	exit;
}


sub get_directories_from_conf
{
	open(CONF, "/usr/local/rserver/trans_recordings.conf") or die "Couldn't open config file";

	my $key;
	while(my $line = <CONF>)
	{
		if ($line =~ /\[(.*)\]/) {
			$key = $1;
			next;
		} else {
			$line =~ /(.*)=(.*)/;
			$options{$key}{$1} = $2;
		}
	}

	close CONF;
	if (!defined($options{directories}{input})
		|| !defined($options{directories}{output})) {
		return 0;
	}
	
	print "[Directories]\n";
	print $options{directories}{input}, " => ", $options{directories}{output}, "\n";
	return 1;
}

# translate recording file under /var/fonality/recordall/recordings/<sid>/ and save the transcoded file to
# /var/fonality/recordall/tivo/<sid>/

sub do_single_trans
{
	my $single_file = shift;
	my($filename, $directories, $suffix) = fileparse($single_file, qr/\.[^.]*~$/);	

	print "Directory: $directories\n";
	print "Filename: $filename\n";
	print "Suffix: $suffix\n";

	my $ext;
	my $found = 0;
	foreach my $format (@ast_format)
	{
		if($suffix =~ /\.\Q$format\E~$/)
		{
                        $found = 1;
                        $ext = $format;
                        last;
		}
	}
	return undef unless($found);

	my $output_filename;
	my $in_or_out;
	if($filename =~ /(.*)\-in$/)
	{
		$output_filename = $1;
		$in_or_out = 1;
	} elsif($filename =~ /(.*)\-out$/) {
		$output_filename = $1;
		$in_or_out = 0;
	} else {
		print "Unknown file: ".$single_file;
		return undef;
	}

	# get the output directory
	my $output_dir = "/var/fonality/recordall/tivo/".$server_id."/";
	my $output_pathname = $output_dir.$output_filename;

	my $save_u = umask();
	umask(0);
	mkpath($output_dir, { verbose => 1, mode => 0777 }) unless(-e $output_dir);
	umask($save_u);

	unlink $output_pathname.".wav";

	# trancode all formats other than alaw & ulaw into ulaw
        my $in_orig_file;
        my $out_orig_file;
        my $command;
        if($in_or_out)
        {
		$in_orig_file = $single_file;
		$out_orig_file = $directories.$output_filename."-out.".$ext."~";
	} else {
		$out_orig_file = $single_file;
		$in_orig_file = $directories.$output_filename."-in.".$ext."~";
	}

	copy($in_orig_file, $output_pathname."-in.".$ext);
	copy($out_orig_file, $output_pathname."-out.".$ext);

	if($ext ne "ulaw" && $ext ne "alaw")
	{
		if(-e $in_orig_file)
		{
                        $command = "/usr/sbin/asterisk -rx \"file convert ".$output_pathname."-in.".$ext." ".$output_pathname."-in.ulaw\"";
                        run_cmd($command) || return;
			unlink $output_pathname."-in.".$ext;
		}

        	if(-e $out_orig_file)
		{
        	        $command = "/usr/sbin/asterisk -rx \"file convert ".$output_pathname."-out.".$ext." ".$output_pathname."-out.ulaw\"";
			run_cmd($command) || return;
			unlink $output_pathname."-out.".$ext;
		}

		$ext = "ulaw";
	}
	
	# merge in & out into one and encode it as mp3 format
	my $in_middle_file = $output_pathname."-in.".$ext;
	my $out_middle_file = $output_pathname."-out.".$ext;
	my $type = $sox_format{$ext};
	my $wav_options = "-t wav -r 8000 -s -w -c 1";

	if( (-e $in_middle_file) && (-e $out_middle_file) ) {
		$command = "soxmix -t $type '$in_middle_file' -t $type '$out_middle_file' $wav_options '$output_pathname'".".wav";
		run_cmd($command) || return;
	} elsif (-e $in_middle_file) {
		$command = "sox -t $type '$in_middle_file' $wav_options '$output_pathname'".".wav";
		run_cmd($command) || return;
	} elsif (-e $out_middle_file) {
		$command = "sox -t $type '$out_middle_file' $wav_options '$output_pathname'".".wav";
		run_cmd($command) || return;
	}

	unlink $in_middle_file;
	unlink $out_middle_file;
	
	print "--- Done: ".$single_file." => ".$output_pathname.".wav"."\n";

	return $output_pathname.".wav";
}

sub traverse_directories
{
	print "[File list]\n";
	find(\&handle_file, $options{directories}{input});
}

sub handle_file
{
	# don't deal with directory
	return if (-d || !-e);

	# check the file format
	my $found = 0;
	my $ext;

	foreach my $format (@ast_format)
	{
		if(/\.\Q$format\E$/)
		{
                        $found = 1;
                        $ext = $format;
                        last;
		}
	}
	return unless($found);

	my $output_filename;
	my $in_or_out;
	if(/(.*)\-in\.\Q$ext\E$/)
	{
		$output_filename = $1;
		$in_or_out = 1;
	} elsif(/(.*)\-out\.\Q$ext\E$/) {
		$output_filename = $1;
		$in_or_out = 0;
	} else {
		print "Unknown file: ".$File::Find::name;
		return;
	}

	# get the output directory
	my $output_pathname;
	my $output_dir;
	foreach my $input_dir ($options{directories}{input})
	{
		my $dir = $File::Find::dir;
		if($dir =~ /^\Q$input_dir\E(.*)/)
		{
			$output_dir = $options{directories}{output}.'/'.$1;
			$output_pathname = $output_dir."/".$output_filename;
			last;
		}
	}	

	my $save_u = umask();
	umask(0);
	mkpath($output_dir, { verbose => 1, mode => 0777 }) unless(-e $output_dir);
	umask($save_u);

	# trancode all formats other than alaw & ulaw into ulaw
        my $in_orig_file;
        my $out_orig_file;
        my $command;
        if($in_or_out)
        {
		$in_orig_file = $File::Find::name;
		$out_orig_file = $File::Find::dir."/".$output_filename."-out.".$ext;
	} else {
		$out_orig_file = $File::Find::name;
		$in_orig_file = $File::Find::dir."/".$output_filename."-in.".$ext;
	}

	if($ext ne "ulaw" && $ext ne "alaw")
	{
		if(-e $in_orig_file)
		{
                        $command = "/usr/sbin/asterisk -rx \"file convert ".$in_orig_file." ".$output_pathname."-in.ulaw\"";
                        run_cmd($command) || return;
		}

        	if(-e $out_orig_file)
		{
        	        $command = "/usr/sbin/asterisk -rx \"file convert ".$out_orig_file." ".$output_pathname."-out.ulaw\"";
			run_cmd($command) || return;
		}

		$ext = "ulaw";
	} else {
		copy($in_orig_file, $output_pathname."-in.".$ext);
		copy($out_orig_file, $output_pathname."-out.".$ext)
	}
	
	# merge in & out into one and encode it as mp3 format
	my $in_middle_file = $output_pathname."-in.".$ext;
	my $out_middle_file = $output_pathname."-out.".$ext;
	my $type = $sox_format{$ext};
	my $rate = 44100;	#it has to be 44100 or above, otherwize red5 won't stream it.

	if( (-e $in_middle_file) && (-e $out_middle_file) ) {
		$command = "soxmix -t $type '$in_middle_file' -t $type '$out_middle_file' -t mp3 -r $rate '$output_pathname.mp3'";
	} elsif (-e $in_middle_file) {
		$command = "sox -t $type $in_middle_file -t mp3 -r $rate $output_pathname.mp3";
	} elsif (-e $out_middle_file) {
		$command = "sox -t $type $out_middle_file -t mp3 -r $rate $output_pathname.mp3";
	}

	run_cmd($command) || return;

	unlink $in_middle_file;
	unlink $out_middle_file;

	if ($options{general}{keep_audio_files} != 1) {
		unlink $in_orig_file;
		unlink $out_orig_file;
	}
	
	# We don't need anymore because we use asterisk to do transcoding and use sox to do mixing and mp3 encoding.

	#$type = $ffmpeg_format{$ext};
	#$command = "ffmpeg -f $type -ar 8000 -ab 64k -i $output_pathname.$ext -ar 44100 -ab 64k $output_pathname.mp3";

	#print "--- Command: ".$command."\n";
	#system($command);
	#if (0 == $?) {
	#	if ($options{general}{keep_audio_files} != 1) {
	#		unlink("$output_pathname.$ext");
	#	}
	#} else {
	#	warn "$command failed: $!";
	#}

	print "--- Done: ".$File::Find::name." => ".$output_pathname.".mp3"."\n";
}

sub run_cmd
{
	my $command = shift;
        print "--- Command: ".$command."\n";
        system($command);
        unless (0 == $?) {
                print  "Failed to run $command: $!";
		return 0;
        }
	return 1;
}
